"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const canvas_1 = require("canvas");
const __1 = require("..");
const function_1 = require("../function");
class NessBuilder {
    canvas;
    context;
    canvasSize = {
        width: 0,
        height: 0
    };
    axis;
    frameCoordinate;
    frameTextCoordinate = { x: 0, y: 0 };
    fontData = [];
    loadingDirection;
    constructor(width, height) {
        this.setCanvas(width, height);
        this.context = this.canvas.getContext('2d');
    }
    ;
    // private async initialize(dataPath: `${string}.ttf`, dataFont: FontOption): Promise<void> {
    //   if (registerFont) {
    //     await this.registerFont(dataPath, dataFont);
    //   }
    //   // Continuer avec l'initialisation du reste de votre objet ici
    // }
    /**
     * New Canvas Dimension
     *
     * @param width x-dimension in pixels
     * @param height y-dimension in pixels
     */
    setCanvas(width, height) {
        this.canvas = new canvas_1.Canvas(width, height);
        this.canvasSize.width = width;
        this.canvasSize.height = height;
        return this;
    }
    ;
    /**
     * Special image background
     * @param color CustomColor<CanvasGradient | CanvasPattern>
     */
    customBackground(color) {
        const canvas = new canvas_1.Canvas(this.canvas.width, this.canvas.height);
        const context = canvas.getContext("2d");
        context.fillStyle = color;
        context.fillRect(0, 0, canvas.width, canvas.height);
        this.setImage(canvas, { sx: 0, sy: 0 });
    }
    ;
    /**
     * canvas outline radius
     *
     * @param radius The radius to set
     * @param outline Line size (default 3)
     * @param color Line color (default #FFFFFF)
     */
    setCornerRadius(radius, outline = 3, color = "#FFFFFF") {
        this.context.lineWidth = outline;
        this.context.strokeStyle = (0, function_1.colorCheck)(color);
        this.context.moveTo(2 + radius, 2);
        this.context.lineTo((this.canvasSize.width - 2) - radius, 2);
        this.context.quadraticCurveTo((this.canvasSize.width - 2), 2, (this.canvasSize.width - 2), 2 + radius);
        this.context.lineTo((this.canvasSize.width - 2), (this.canvasSize.height - 2) - radius);
        this.context.quadraticCurveTo((this.canvasSize.width - 2), (this.canvasSize.height - 2), (this.canvasSize.width - 2) - radius, (this.canvasSize.height - 2));
        this.context.lineTo(2 + radius, (this.canvasSize.height - 2));
        this.context.quadraticCurveTo(2, (this.canvasSize.height - 2), 2, (this.canvasSize.height - 2) - radius);
        this.context.lineTo(2, 2 + radius);
        this.context.quadraticCurveTo(2, 2, 2 + radius, 2);
        this.context.stroke();
        this.context.clip();
        return this;
    }
    ;
    /**
     * Sets the canvas background.
     * @param imageColor The image to set (no link, use loadImage() from canvas) or a custom color (Valid syntaxes: #hex | rgb | rgba | colorName | CanvasGradient | CanvasPattern)
     */
    setBackground(imageColor) {
        if (typeof imageColor == "object" && !(imageColor instanceof canvas_1.CanvasPattern) && !(imageColor instanceof canvas_1.CanvasGradient)) {
            this.setImage(imageColor, { sx: 0, sy: 0, sWidth: this.canvas.width, sHeight: this.canvas.height });
        }
        else if ((imageColor instanceof canvas_1.CanvasPattern) || (imageColor instanceof canvas_1.CanvasGradient)) {
            this.customBackground(imageColor);
        }
        else {
            this.context.fillStyle = (0, function_1.colorCheck)(imageColor);
            this.context.fillRect(0, 0, this.canvas.width, this.canvas.height);
        }
        return this;
    }
    ;
    /**
     * Draw an image to S coordinates with D dimensions
     *
     * @param image The image to set (no link, use loadImage() from canvas)
     * @param imageOption Source image coordinates to draw in the context of Canvas
     * @param locationOption Modify image coordinates to draw in the context of Canvas
     */
    setImage(image, imageOption, locationOption) {
        if (locationOption) {
            this.context.drawImage(image, locationOption.dx, locationOption.dy, locationOption.dWidth, locationOption.dHeight, imageOption.sx, imageOption.sy, imageOption.sWidth, imageOption.sHeight);
        }
        else if (imageOption.sWidth && imageOption.sHeight) {
            this.context.drawImage(image, imageOption.sx, imageOption.sy, imageOption.sWidth, imageOption.sHeight);
        }
        else
            this.context.drawImage(image, imageOption.sx, imageOption.sy);
        return this;
    }
    ;
    /**
     * Sets a predefined frame
     *
     * @param typeShape Frame format
     * @param coordinate Coordinate X and Y from upper left corner of the frame
     * @param size Frame size
     * @param options Frame configuration
     */
    setFrame(shape, frame, options) {
        this.context.save();
        this.context.strokeStyle = options.color ? (0, function_1.colorCheck)(options.color) : "#FF0000";
        this.context.lineWidth = options.lineWidth ? options.lineWidth : 3;
        this.setShape(shape, frame);
        // this.context.closePath();
        this.context.stroke();
        this.context.clip();
        if (options.type == "Image") {
            return this.setFrameBackground(options.content);
        }
        else if (options.type == "Text") {
            if (frame.rotate)
                this.setRotation(frame.x, frame.y, -frame.rotate);
            const textOptions = options.textOptions;
            if (textOptions?.backgroundColor) {
                this.context.fillStyle = (0, function_1.colorCheck)(textOptions.backgroundColor);
                this.context.fill();
            }
            ;
            const defaultTextOption = {
                size: 30,
            };
            const textOption = textOptions?.size ? { ...defaultTextOption, ...textOptions } : { ...textOptions, ...defaultTextOption };
            this.setText(options.content.toString(), { x: this.frameTextCoordinate.x, y: this.frameTextCoordinate.y }, { size: textOption.size, font: textOptions?.font ? textOptions?.font : "*Arial", color: textOptions?.color ? textOptions.color : "#FFFFFF", textAlign: textOptions?.textAlign ? textOptions.textAlign : "center", textBaseline: textOptions?.textBaseline ? textOptions.textBaseline : "middle" });
            this.restore();
            return this;
        }
        else if (options.type == "Color") {
            this.context.fillStyle = (0, function_1.colorCheck)(options.content);
            this.context.fill();
            this.restore();
            return this;
        }
        else {
            this.restore();
            return this;
        }
    }
    ;
    // Mise en palce d'un cadre
    setShape(shape, frame) {
        const axis = this.getAxis(frame);
        // frame.x = axis.x, frame.y = axis.y;
        this.frameCoordinate = { ...frame, ...axis };
        if (frame.rotate)
            this.setRotation(axis.x, axis.y, frame.rotate);
        this.context.beginPath();
        const radius = (["Square", "Rectangle"].includes(shape) && frame.QuadrilateralOption) ? frame.QuadrilateralOption.radius : 15;
        const sizeX = frame.QuadrilateralOption?.width && shape.includes("Rectangle") ? frame.QuadrilateralOption.width : frame.size;
        const sizeY = frame.QuadrilateralOption?.height && shape.includes("Rectangle") ? frame.QuadrilateralOption.height : frame.size;
        const r = axis.x + sizeX;
        const b = axis.y + sizeY;
        switch (shape) {
            case "Square": {
                this.context.moveTo(axis.x + radius - frame.size, axis.y - frame.size);
                this.context.lineTo(r - radius, axis.y - frame.size);
                this.context.quadraticCurveTo(r, axis.y - frame.size, r, axis.y + radius - frame.size);
                this.context.lineTo(r, axis.y + frame.size - radius);
                this.context.quadraticCurveTo(r, b, r - radius, b);
                this.context.lineTo(axis.x + radius - frame.size, b);
                this.context.quadraticCurveTo(axis.x - frame.size, b, axis.x - frame.size, b - radius);
                this.context.lineTo(axis.x - frame.size, axis.y + radius - frame.size);
                this.context.quadraticCurveTo(axis.x - frame.size, axis.y - frame.size, axis.x + radius - frame.size, axis.y - frame.size);
                this.frameTextCoordinate = { x: axis.x, y: axis.y };
                break;
            }
            case "Rectangle": {
                const sizeX = frame.QuadrilateralOption?.width ? frame.QuadrilateralOption.width : axis.x;
                const r = axis.x + sizeX;
                this.context.moveTo(axis.x + radius - sizeX, axis.y - sizeY);
                this.context.lineTo(r - radius, axis.y - sizeY);
                this.context.quadraticCurveTo(r, axis.y - sizeY, r, axis.y + radius - sizeY);
                this.context.lineTo(r, axis.y + sizeY - radius);
                this.context.quadraticCurveTo(r, b, r - radius, b);
                this.context.lineTo(axis.x + radius - sizeX, b);
                this.context.quadraticCurveTo(axis.x - sizeX, b, axis.x - sizeX, b - radius);
                this.context.lineTo(axis.x - sizeX, axis.y + radius - sizeY);
                this.context.quadraticCurveTo(axis.x - sizeX, axis.y - sizeY, axis.x + radius - sizeX, axis.y - sizeY);
                this.frameTextCoordinate = { x: axis.x, y: axis.y };
                break;
            }
            case "Circle": {
                this.context.arc(axis.x, axis.y, frame.size, 0, 2 * Math.PI);
                this.frameTextCoordinate = { x: axis.x, y: axis.y };
                break;
            }
            default:
                {
                    const angle = (Math.PI * 2) / __1.ShapeEnum[shape];
                    for (let i = 0; i <= __1.ShapeEnum[shape]; i++) {
                        const x = axis.x + frame.size * Math.cos(angle * i);
                        const y = axis.y + frame.size * Math.sin(angle * i);
                        if (i === 0)
                            this.context.moveTo(x, y);
                        else
                            this.context.lineTo(x, y);
                    }
                    ;
                    this.frameTextCoordinate = { x: axis.x, y: axis.y };
                    break;
                }
                ;
            // case "SymmetricalStar": {
            //   let rot = Math.PI / 2 * 3;
            //   const spikes = options.radPik;
            //   const step = Math.PI / spikes;
            //   this.context.beginPath();
            //   for (let i = 0; i < spikes; i++) {
            //     let x = axis.x + frame.size / 2 + Math.cos(rot) * frame.size / 4;
            //     let y = axis.y + frame.size / 2 + Math.sin(rot) * frame.size / 4;
            //     this.context.lineTo(x, y);
            //     rot += step;
            //     x = axis.x + frame.size / 2 + Math.cos(rot) * frame.size / 2;
            //     y = axis.y + frame.size / 2 + Math.sin(rot) * frame.size / 2;
            //     this.context.lineTo(x, y);
            //     rot += step;
            //   }
            //   this.frameTextCoordinate.x = axis.x + frame.size / 2 + Math.cos(rot);
            //   this.frameTextCoordinate.y = axis.y + frame.size / 2 + Math.cos(rot);
            //   this.context.closePath();
            //   break;
            // };
        }
        ;
        // this.context.closePath();
        return this;
    }
    ;
    // Tourner l'élément
    setRotation(centerX, centerY, angle) {
        this.context.translate(centerX, centerY);
        this.context.rotate((angle * Math.PI) / 180);
        this.context.translate(-centerX, -centerY);
    }
    ;
    // Définition du background du cadre
    setFrameBackground(image) {
        this.setRotation(this.frameCoordinate.x, this.frameCoordinate.y, -this.frameCoordinate.rotate);
        while (this.frameCoordinate.rotate > 45 || this.frameCoordinate.rotate < -45) {
            if (this.frameCoordinate.rotate > 0)
                this.frameCoordinate.rotate -= 45;
            else
                this.frameCoordinate.rotate = -this.frameCoordinate.rotate;
        }
        const size = this.frameCoordinate.rotate ? 0.009 * this.frameCoordinate.rotate : 0;
        const size2 = this.frameCoordinate.rotate ? 0.018 * this.frameCoordinate.rotate : 0;
        if (typeof image == "object" && !(image instanceof canvas_1.CanvasPattern) && !(image instanceof canvas_1.CanvasGradient)) {
            this.setImage(image, { sx: this.frameCoordinate.x - this.frameCoordinate.size * (1 + size), sy: this.frameCoordinate.y - this.frameCoordinate.size * (1 + size), sWidth: this.frameCoordinate.size * (2 + size2), sHeight: this.frameCoordinate.size * (2 + size2) });
        }
        else {
            this.customBackground(image);
        }
        this.context.restore();
        return this;
    }
    ;
    /**
     * Set text to canvas
     *
     * @param text Text to write
     * @param coordinate Text location
     * @param option Text option
     */
    setText(text, coordinate, option) {
        option.font ? this.setFont(option.font, option.size) : this.setFont("*Sans", option.size);
        this.context.fillStyle = option.color ? (0, function_1.colorCheck)(option.color) : "#FFF";
        this.context.textAlign = option.textAlign;
        this.context.textBaseline = option.textBaseline;
        option.stroke ? this.context.strokeText(text, coordinate.x, coordinate.y) : this.context.fillText(text, coordinate.x, coordinate.y);
        return this;
    }
    ;
    /**
     * Change font to use
     *
     * @param name Add * to use the system font (*Arial) or CustomFont (RegisterFont)
     * @param size Font size
     */
    setFont(name, size) {
        if (this.getFont(name)) {
            this.context.font = `${size}px ${name.replace("*", "")}`;
        }
        ;
        return this;
    }
    ;
    // Récupère une police enregistrer hormis celle du système
    getFont(name, option) {
        const dataF = this.fontData.find(x => x.font.family === name);
        if (option?.path) {
            const dataP = this.fontData.find(x => x.file === option.path);
            if (dataP) {
                console.error(`\x1b[33mRegisterFont: \x1b[32mThis file (\x1b[31m${option.path.replace(/^.*[\\/]/, "")}\x1b[32m) has already been register for \x1b[35m${dataP.font.family}\x1b[0m`);
                return false;
            }
            else if (dataF) {
                console.error(`\x1b[33mRegisterFont: \x1b[32mThis name (\x1b[31m${dataF.font.family}\x1b[32m) has already been register for \x1b[35m${dataF.file.replace(/^.*[\\/]/, "")}\x1b[0m`);
                return false;
            }
            return true;
        }
        else {
            if (!dataF && !name.startsWith('*')) {
                console.error(`\x1b[33mRegisterFont: \x1b[31m${name} \x1b[32mis not part of the police register\x1b[0m`);
                return false;
            }
            return true;
        }
    }
    /**
     * register a new font
     *
     * @param path Path to font file (file.ttf)
     * @param option Font default settings
     */
    registerFont(font) {
        font.forEach(e => {
            if (this.getFont(e.font.family, { path: e.file })) {
                (0, canvas_1.registerFont)(e.file, e.font);
                this.fontData.push({ file: e.file, font: e.font });
                console.log(`\x1b[33mRegisterFont: \x1b[35m${e.font.family} \x1b[30m| \x1b[32msize - \x1b[35m${e.font.size ? e.font.size + "px" : "Default"} \x1b[30m| \x1b[32mStyle - \x1b[35m${e.font.style}\x1b[0m`);
            }
            ;
        });
        return this;
    }
    ;
    // Restore la sélection
    restore() {
        this.context.restore();
        return this;
    }
    ;
    /**
     * Set progress bar
     *
     * @param location Coordinate to set ExpBar
     * @param size Size of the first progression bar
     * @param radius Radius to set
     * @param progress Progression of the second progression bar 0 - 100%
     * @param color Text color. White color is used by Default
     */
    setExp(option, progress, color) {
        const axis = this.getAxis({ x: option.x, y: option.y, size: option.width }, option.height);
        this.setRotation(axis.x, axis.y, option.rotate);
        this.context.save();
        this.context.beginPath();
        this.context.strokeStyle = color?.outlineColor1 ? (0, function_1.colorCheck)(color.outlineColor1) : "#FF0000";
        this.context.lineWidth = 2;
        this.context.globalAlpha = option.alphat ? option.alphat / 100 : 0.3;
        // Barre N°1
        this.context.moveTo(axis.x - option.width, axis.y - option.height);
        this.context.lineTo(axis.x, axis.y - option.height);
        this.context.quadraticCurveTo(axis.x + option.radius, axis.y - option.height, axis.x + option.radius, axis.y - (option.height / 2));
        this.context.quadraticCurveTo(axis.x + option.radius, axis.y, axis.x, axis.y);
        this.context.lineTo(axis.x - option.width, axis.y);
        this.context.quadraticCurveTo(axis.x - option.width - option.radius, axis.y, axis.x - option.width - option.radius, axis.y - (option.height / 2));
        this.context.quadraticCurveTo(axis.x - option.width - option.radius, axis.y - option.height, axis.x - option.width, axis.y - option.height);
        this.context.fillStyle = color?.color1 ? (0, function_1.colorCheck)(color.color1) : "#FFFFFF";
        this.context.fill();
        this.context.stroke();
        if ((progress * 100) / option.width < 6.8) {
            this.context.clip();
        }
        this.context.closePath();
        const progressFill = ((progress * option.width) / 100) - axis.x + option.x;
        this.context.globalAlpha = 1;
        // Barre N°2
        this.context.beginPath();
        this.context.strokeStyle = color?.outlineColor2 ? (0, function_1.colorCheck)(color.outlineColor2) : "#000000";
        this.context.lineWidth = 2;
        this.context.moveTo(axis.x - option.width, axis.y - option.height);
        this.context.lineTo(axis.x + progressFill, axis.y - option.height);
        this.context.quadraticCurveTo(axis.x + option.radius + progressFill, axis.y - option.height, axis.x + option.radius + progressFill, axis.y - (option.height / 2));
        this.context.quadraticCurveTo(axis.x + option.radius + progressFill, axis.y, axis.x + progressFill, axis.y);
        this.context.lineTo(axis.x - option.width, axis.y);
        this.context.quadraticCurveTo(axis.x - option.width - option.radius, axis.y, axis.x - option.width - option.radius, axis.y - (option.height / 2));
        this.context.quadraticCurveTo(axis.x - option.width - option.radius, axis.y - option.height, axis.x - option.width, axis.y - option.height);
        // this.context.moveTo(axis.x, axis.y);
        // this.context.lineTo(axis.x + progressFill, axis.y);
        // this.context.quadraticCurveTo(axis.x + progressFill + option.radius, axis.y, axis.x + progressFill + option.radius, axis.y + (option.height /2));
        // this.context.quadraticCurveTo(axis.x + progressFill + option.radius, axis.y + option.height, axis.x + progressFill, axis.y + option.height);
        // this.context.lineTo(axis.x, axis.y + option.height);
        // this.context.quadraticCurveTo(axis.x - option.radius, axis.y + option.height, axis.x - option.radius, axis.y + (option.height /2));
        // this.context.quadraticCurveTo(axis.x - option.radius, axis.y, axis.x, axis.y);
        this.context.fillStyle = color?.color2 ? (0, function_1.colorCheck)(color.color2) : "#BB00FF";
        this.context.fill();
        this.context.stroke();
        this.context.restore();
        return this;
    }
    ;
    setLoading(shape, option) {
        const progress = option.progress / 100;
        // const endAngle = progress * 2 * Math.PI;
        // -1.57 < 0 > 1.57
        // const startAngle = 3.14
        // const startAngle = 0
        // const startHour = 8.24
        const regex = /^(\d{1,2})h(\d{1})(?!\d)$/;
        const match = option.fill.start?.match(regex);
        if (match) {
            option.fill.start = `${match[1]}h0${match[2]}`;
        }
        ;
        const startAngle = (((option.fill.start ? parseFloat(option.fill.start?.replace("h", ".")) : 0) - 3) / 12) * 2 * Math.PI;
        const endAngle = (startAngle + progress * 2 * Math.PI) % (2 * Math.PI);
        let rotate = 0;
        if (option.fill.start) {
            this.setLoadingDirection(option.fill.start);
        }
        ;
        this.context.save();
        this.context.beginPath();
        this.context.lineWidth = option.outline.width ? option.outline.width : 5;
        this.setShape(shape, option);
        rotate = option.rotate;
        option.rotate = undefined;
        this.context.globalAlpha = 0.38;
        this.context.fillStyle = option.color;
        this.setRotation(option.x, option.y, this.loadingDirection + -rotate);
        this.context.fill();
        this.setRotation(option.x, option.y, rotate + -this.loadingDirection);
        this.context.closePath();
        this.context.clip();
        this.context.beginPath();
        this.context.strokeStyle = option.color ? (0, function_1.colorCheck)(option.color) : "#FF0000";
        this.context.fillStyle = option.color ? (0, function_1.colorCheck)(option.color) : "#FF0000";
        this.context.globalAlpha = 1;
        if (option.fill.type == "Circle") {
            this.context.moveTo(option.x, option.y);
            shape.includes("Rectangle") ? option.size = option.QuadrilateralOption.width >= option.QuadrilateralOption.height ? option.QuadrilateralOption.width : option.QuadrilateralOption.height : undefined;
            this.context.arc(option.x, option.y, option.size * 1.5, startAngle, endAngle);
        }
        else {
            let axis = this.getAxis(option);
            const radius = ["Square", "Rectangle"].includes(shape) ? option.QuadrilateralOption.radius : 0;
            while (rotate > 45 || rotate < -45) {
                if (rotate > 0)
                    rotate -= 45;
                else
                    rotate = -rotate;
            }
            ;
            // D = C √2
            // D = (option.size + radius) √2
            // D <=> max
            // if rotate => D = ((rotate * (option.size + radius)) /45) √2
            const size = rotate ? 0.45 * rotate + radius : 0;
            // const size2 = rotate? 0.45 * rotate+radius : 0;
            // const max = (option.progress * (option.size*2 + size))/100;
            let max = ((option.progress * (option.size + radius)) / 100) * 2;
            // console.log(max)
            let x = axis.x - option.size - size, y = axis.y - option.size - size;
            if (["Rectangle" || "Square"].includes(shape)) {
                if (shape.includes("Rectangle")) {
                    x = axis.x - option.QuadrilateralOption.width;
                    y = axis.y - option.QuadrilateralOption.height;
                    option.size = (option.QuadrilateralOption.height) * 2;
                    max = ((option.progress * (option.QuadrilateralOption.width)) / 100) * 2;
                }
                this.setRotation(axis.x, axis.y, this.loadingDirection + -rotate);
                this.context.moveTo(x - size, y - size);
                this.context.lineTo(x + max, y - size);
                this.context.lineTo(x + max, axis.y + option.size + size);
                this.context.lineTo(x - size, axis.y + option.size + size);
                this.setRotation(axis.x, axis.y, rotate + -this.loadingDirection);
            }
            else {
                this.setRotation(axis.x, axis.y, this.loadingDirection + -rotate);
                this.context.moveTo(x - size, y - size * 4);
                this.context.lineTo(x + max, y - size * 4);
                this.context.lineTo(x + max, axis.y + option.size * 4 + size);
                this.context.lineTo(x - size, axis.y + option.size * 4 + size);
                this.setRotation(axis.x, axis.y, rotate + -this.loadingDirection);
            }
        }
        this.setRotation(option.x, option.y, this.loadingDirection + -rotate);
        this.context.fill();
        this.setRotation(option.x, option.y, rotate + -this.loadingDirection);
        this.context.closePath();
        this.context.beginPath();
        this.context.strokeStyle = option.outline.color ? (0, function_1.colorCheck)(option.outline.color) : "#FF0000";
        this.context.fillStyle = "rgba(0,0,0,0)";
        this.setShape(shape, option);
        this.context.fill();
        this.context.stroke();
        this.context.closePath();
        this.restore();
        return this;
    }
    ;
    setLoadingDirection(direction) {
        if (direction == "DownRightToUpLeft") {
            this.loadingDirection = -135;
        }
        else if (direction == "DownToUp") {
            this.loadingDirection = -90;
        }
        else if (direction == "DownLeftToUpRight") {
            this.loadingDirection = -45;
        }
        else if (direction == "LeftToRight") {
            this.loadingDirection = 0;
        }
        else if (direction == "UpLeftToDownRight") {
            this.loadingDirection = 45;
        }
        else if (direction == "UpToDown") {
            this.loadingDirection = 90;
        }
        else if (direction == "UpRightToDownLeft") {
            this.loadingDirection = 135;
        }
        else if (direction == "RightToLeft") {
            this.loadingDirection = 180;
        }
    }
    setAxis(axis) {
        this.axis = axis;
        return this;
    }
    ;
    // Change l'axe par default du cadre donné
    getAxis(frame, height) {
        const heightSize = height ? height : frame.size;
        if (this.axis == "TopLeft")
            return { x: frame.x - frame.size, y: frame.y - heightSize };
        else if (this.axis == "TopCenter")
            return { x: frame.x, y: frame.y - heightSize };
        else if (this.axis == "TopRight")
            return { x: frame.x + frame.size, y: frame.y - heightSize };
        else if (this.axis == "BottomLeft")
            return { x: frame.x - frame.size, y: frame.y + heightSize };
        else if (this.axis == "BottomCenter")
            return { x: frame.x, y: frame.y + heightSize };
        else if (this.axis == "BottomRight")
            return { x: frame.x + frame.size, y: frame.y + heightSize };
        else if (this.axis == "Left")
            return { x: frame.x - frame.size, y: frame.y };
        else if (this.axis == "Right")
            return { x: frame.x + frame.size, y: frame.y };
        else
            return { x: frame.x, y: frame.y };
    }
    ;
    /**
     * Return canvas Buffer
     */
    toBuffer(ext) {
        if (ext == "jpeg" || ext == "jpg")
            return this.canvas.toBuffer("image/jpeg");
        else if (ext == "png")
            return this.canvas.toBuffer("image/png");
        else if (ext == "raw")
            return this.canvas.toBuffer("raw");
        else if (ext == "pdf")
            return this.canvas.toBuffer("application/pdf");
        else
            return this.canvas.toBuffer();
    }
    ;
    /**
     * Generated image from canvas
     *
     * @param location Image Generation Path
     * @param name Image name
     * @param type Image extention
     */
    generatedTo(location, name, type) {
        if (type == "png" || type == "jpeg" || type == "jpg")
            (0, fs_1.writeFileSync)(`${location}/${name}.${type}`, this.toBuffer(type));
        else
            (0, fs_1.writeFileSync)(`${location}/${name}.${type}`, this.toBuffer());
    }
    ;
    /**
     * Returns a base64 encoded string
     */
    toDataURL() {
        return this.canvas.toDataURL();
    }
    ;
}
exports.default = NessBuilder;
