"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.colorCheck = void 0;
const _1 = require(".");
const canvas_1 = require("canvas");
function colorCheck(color) {
    function isHexColorCode(input) {
        const hexColorCodeRegex = /^#?([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3}|[A-Fa-f0-9]{4}|[A-Fa-f0-9]{8})$/;
        return hexColorCodeRegex.test(input);
    }
    function isRGBColorCode(input) {
        const rgbColorCodeRegex = /^rgb\(\s*\d+\s*,\s*\d+\s*,\s*\d+\s*\)$/i;
        return rgbColorCodeRegex.test(input);
    }
    function isRGBAColorCode(input) {
        const rgbaColorCodeRegex = /^rgba\(\s*\d+\s*,\s*\d+\s*,\s*\d+\s*,\s*[\d\.]+\s*\)$/i;
        return rgbaColorCodeRegex.test(input);
    }
    function isPredefinedColorName(input) {
        const predefinedColors = [
            "White", "Yellow", "Blue", "Red", "Green", "Black", "Brown", "Ivory", "Teal", "Silver", "Purple", "Navy", "Gray", "Orange", "Maroon", "Aquamarine", "Coral", "Fuchsia", "Wheat", "Lime", "Crimson", "Khaki", "HotPink", "Magenta", "Plum", "Olive", "Cyan"
        ];
        return predefinedColors.includes(input);
    }
    if (!isHexColorCode(color) && !isRGBColorCode(color) && !isRGBAColorCode(color) && !isPredefinedColorName(color) && !(color instanceof canvas_1.CanvasGradient) && !(color instanceof canvas_1.CanvasPattern)) {
        console.error(new Error(`\x1b[33mColor Checker: \x1b[31m${color} \x1b[32mIs not valid\n\x1b[36mValid syntaxes: \x1b[32m#hex \x1b[36m| \x1b[32mrgb \x1b[36m| \x1b[32mrgba \x1b[36m| \x1b[32mcolorName \x1b[36m| \x1b[32mCanvasGradient \x1b[36m| \x1b[32mCanvasPattern\x1b[0m`));
    }
    else if (isRGBAColorCode(color) || isRGBColorCode(color)) {
        return (0, _1.rgbToHex)(color);
    }
    else
        return color;
}
exports.colorCheck = colorCheck;
